'' =================================================================================================
''
''   File....... jm_1-wire.spin
''   Purpose.... Essential 1-Wire interface routines
''   Author..... Jon "JonnyMac" McPhalen (aka Jon Williams)
''               Copyright (c) 2009-2011 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started.... 29 JUL 2009
''   Updated.... 17 MAR 2011
''
'' =================================================================================================


var

  long  owcmd                                                   ' command to 1-Wire interface
  long  owio                                                    ' data in/out
  long  owpin                                                   ' pin used
  long  us001                                                   ' ticks per microsecond   

  long  cog                                                     ' cog used by 1W driver
  

pub init(pin) 

'' Starts 1-Wire object

  finalize                                                      ' stop if already running

  if (pin => 0) and (pin =< 27)                                 ' protect rx, tx, i2c
    owpin := pin                                                ' set pin to use
    us001 := clkfreq / 1_000_000                                ' set ticks per microsecond

    dira[pin] := 0                                              ' float OW pin
    owcmd := 0                                                  ' clear command                                                        
    cog := cognew(@onewire, @owcmd) + 1                         ' start cog

  else
    cog := -1                                                   ' bad pin 

  return (cog > 0)


pub finalize

'' Stops 1-Wire driver; frees a cog 

  if cog
    cogstop(cog~ - 1)


pub reset

'' Resets 1-Wire buss; returns buss status
''
''   %00 = buss short
''   %01 = bad response; possible interference on buss
''   %10 = good buss & presence detection
''   %11 = no device 

  owio := %11                                                   ' assume no device
  owcmd := 1                                                    ' execut command
  repeat while owcmd                                            ' wait until finished
  
  return owio                                                   ' return buss status


pub write(b)

'' Write byte b to 1-Wire buss

  owio := b & $FF                                               ' cleanup byte
  owcmd := 2                                                    ' execute command
  repeat while owcmd                                            ' wait until finished


pub read

'' Reads byte from 1-Wire buss

  owio := 0                                                     ' clear old result
  owcmd := 3                                                    ' execute command  
  repeat while owcmd                                            ' wait until finished 

  return (owio & $FF)                                           ' return 1-wire byte  


pub rdbit

'' Reads bit from 1-Wire buss
'' -- useful for monitoring device busy status

  owio := 0                                                     ' clear old result     
  owcmd := 4                                                    ' execute command      
  repeat while owcmd                                            ' wait until finished  
                                                                                       
  return (owio & 1)                                             ' return 1-wire bit   


pub crc8(pntr, n)

'' Returns CRC8 of n bytes at pntr
'' -- interface to PASM code by Cam Thompson

  owio := pntr                                                  ' store address of array
  owcmd := (n << 8) + 5                                         ' pack count into command
  repeat while owcmd                                            ' wait until finished   

  return owio                                                   ' return crc


pub crc8x(pntr, n) | crc, b

'' Returns CRC8 of n bytes at pntr
'' -- code by Micah Dowty

  crc := 0
  repeat n
    b := byte[pntr++]
    repeat 8
      if (crc ^ b) & 1
        crc := (crc >> 1) ^ $8C
      else
        crc >>= 1
      b >>= 1  

  return crc


dat

                        org     0

onewire                 mov     dira, #0                        ' all inputs
                        mov     outa, #0

                        mov     tmp1, par                       ' get start of structure
                        mov     cmdpntr, tmp1                   ' save hub addr of command
                        add     tmp1, #4
                        mov     iopntr, tmp1                    ' save hub addr of io value
                        add     tmp1, #4
                        rdlong  tmp2, tmp1                      ' read pin #
                        mov     owmask, #1                      ' create pin mask
                        shl     owmask, tmp2
                        add     tmp1, #4
                        rdlong  ustix, tmp1                     ' read ticks per microseconds
                        
owmain                  rdlong  tmp1, cmdpntr           wz      ' get command
        if_z            jmp     #owmain                         ' wait for valid command
                        mov     bytecount, tmp1                 ' make copy (for crc byte count)
                        and     tmp1, #$FF                      ' strip off count

                        min     tmp1, #0                        ' limit range
                        max     tmp1, #6                        ' commands + 1
                        add     tmp1, #jmptable                 ' prep for jump
                        jmp     tmp1                            ' do it                     

jmptable                jmp     #cmdexit                        ' invalid command
do_cmd1                 jmp     #owreset                        ' reset 1-wire buss                       
do_cmd2                 jmp     #owwrbyte                       ' write a byte
do_cmd3                 jmp     #owrdbyte                       ' read a byte
do_cmd4                 jmp     #owrdbit                        ' read a bit
do_cmd5                 jmp     #owcalccrc                      ' calculate CRC

cmdexit                 wrlong  ZERO, cmdpntr                   ' clear old command 
                        jmp     #owmain


                        
' -----------------
' Reset 1-Wire buss
' -----------------
'
' %00 = buss short
' %01 = bad response; possible interference on buss
' %10 = good buss & presence detection
' %11 = no device

owreset                 mov     value, #%11                     ' assume no device
                        or      dira, owmask                    ' buss low
                        mov     usecs, #500                     ' reset pulse                     
                        call    #pauseus
                        andn    dira, owmask                    ' release buss
                        mov     usecs, #5
                        call    #pauseus
                        test    owmask, ina             wc      ' sample for short, 1W -> C
                        muxc    value, #%10                     ' C -> value.1
                        mov     usecs, #70
                        call    #pauseus
                        test    owmask, ina             wc      ' sample for presence, 1W -> C
                        muxc    value, #%01                     ' C -> value.0                        
                        mov     usecs, #400
                        call    #pauseus
                        wrlong  value, iopntr                   ' update hub
                        wrlong  ZERO, cmdpntr
                        jmp     #owmain                          
                        

                        
' -------------------------
' Write byte to 1-Wire buss
' -------------------------
'
owwrbyte                rdlong  value, iopntr                   ' get byte from hub
                        mov     bitcount, #8                    ' write 8 bits
wrloop                  shr     value, #1               wc      ' value.0 -> C, value >>= 1
        if_c            mov     usecs, #6                       ' write 1
        if_nc           mov     usecs, #60                      ' write 0
                        or      dira, owmask                    ' pull buss low 
                        call    #pauseus
                        andn    dira, owmask                    ' release buss
        if_c            mov     usecs, #64                      ' pad for 1
        if_nc           mov     usecs, #10                      ' pad for 0
                        call    #pauseus
                        djnz    bitcount, #wrloop               ' all bits done?
                        wrlong  ZERO, cmdpntr                   ' yes, update hub
                        jmp     #owmain

                        

' --------------------------
' Read byte from 1-Wire buss
' --------------------------
'
owrdbyte                mov     bitcount, #8                    ' read 8 bits
                        mov     value, #0                       ' clear workspace
rdloop                  or      dira, owmask                    ' buss low
                        mov     usecs, #6 
                        call    #pauseus
                        andn    dira, owmask                    ' release buss
                        mov     usecs, #9                       ' hold-off before sample
                        call    #pauseus
                        test    owmask, ina             wc      ' sample buss, 1W -> C
                        shr     value, #1                       ' value >>= 1
                        muxc    value, #%1000_0000              ' C -> value.7
                        mov     usecs, #55                      ' finish read slot
                        call    #pauseus                          
                        djnz    bitcount, #rdloop               ' all bits done?
                        wrlong  value, iopntr                   ' yes, update hub
                        wrlong  ZERO, cmdpntr                       
                        jmp     #owmain

                        

' -------------------------
' Read bit from 1-Wire buss
' -------------------------
'
owrdbit                 mov     value, #0                       ' clear workspace
                        or      dira, owmask                    ' buss low
                        mov     usecs, #6
                        call    #pauseus
                        andn    dira, owmask                    ' release buss
                        mov     usecs, #9                       ' hold-off before sample
                        call    #pauseus
                        test    owmask, ina             wc      ' sample buss, 1W -> C 
                        muxc    value, #1                       ' C -> value.0                        
                        mov     usecs, #55                      ' finish read slot
                        call    #pauseus
                        wrlong  value, iopntr                   ' update hub
                        wrlong  ZERO, cmdpntr                       
                        jmp     #owmain


                        
' ----------------------
' Calculate CRC8
' * code by Cam Thompson 
' ----------------------
'
owcalccrc               mov     value, #0                       ' clear workspace 
                        shr     bytecount, #8           wz      ' clear command from count
        if_z            jmp     #savecrc
                        rdlong  hubpntr, iopntr                 ' get address of array

crcbyte                 rdbyte  tmp1, hubpntr                   ' read byte from array
                        add     hubpntr, #1                     ' point to next
                        mov     bitcount, #8
                        
crcbit                  mov     tmp2, tmp1                      ' x^8 + x^5 + x^4 + 1  
                        shr     tmp1, #1
                        xor     tmp2, value
                        shr     value, #1
                        shr     tmp2, #1                wc
        if_c            xor     value, #$8C
                        djnz    bitcount, #crcbit
                        djnz    bytecount, #crcbyte

savecrc                 wrlong  value, iopntr                   ' update hub
                        wrlong  ZERO, cmdpntr                       
                        jmp     #owmain                


                        
' ---------------------
' Pause in microseconds
' ---------------------
' 
pauseus                 mov     ustimer, cnt                    ' sync with system clock
                        sub     ustimer, #24                    ' account for setup/call/return
                        add     ustimer, ustix                  ' set timer for 1us
usloop                  waitcnt ustimer, ustix                  ' wait and reload
                        djnz    usecs, #usloop                  ' update delay count
pauseus_ret             ret


' --------------------------------------------------------------------------------------------------

ZERO                    long    0

cmdpntr                 res     1                               ' hub address of command
iopntr                  res     1                               ' hub address of io value
owmask                  res     1                               ' mask of pin used
ustix                   res     1                               ' clock ticks in 1us

value                   res     1 
bitcount                res     1
bytecount               res     1
hubpntr                 res     1
usecs                   res     1
ustimer                 res     1

tmp1                    res     1
tmp2                    res     1

                        fit
                        

dat

{{

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 

}}                   